/*******************************************************************************
* File Name: cADC.h
* Version 2.20
*
*  Description:
*   Provides the function and constant definitions for the clock component.
*
*  Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_CLOCK_cADC_H)
#define CY_CLOCK_cADC_H

#include <cytypes.h>
#include <cyfitter.h>


/***************************************
* Conditional Compilation Parameters
***************************************/

/* Check to see if required defines such as CY_PSOC5LP are available */
/* They are defined starting with cy_boot v3.0 */
#if !defined (CY_PSOC5LP)
    #error Component cy_clock_v2_20 requires cy_boot v3.0 or later
#endif /* (CY_PSOC5LP) */


/***************************************
*        Function Prototypes
***************************************/

void cADC_Start(void) ;
void cADC_Stop(void) ;

#if(CY_PSOC3 || CY_PSOC5LP)
void cADC_StopBlock(void) ;
#endif /* (CY_PSOC3 || CY_PSOC5LP) */

void cADC_StandbyPower(uint8 state) ;
void cADC_SetDividerRegister(uint16 clkDivider, uint8 restart) 
                                ;
uint16 cADC_GetDividerRegister(void) ;
void cADC_SetModeRegister(uint8 modeBitMask) ;
void cADC_ClearModeRegister(uint8 modeBitMask) ;
uint8 cADC_GetModeRegister(void) ;
void cADC_SetSourceRegister(uint8 clkSource) ;
uint8 cADC_GetSourceRegister(void) ;
#if defined(cADC__CFG3)
void cADC_SetPhaseRegister(uint8 clkPhase) ;
uint8 cADC_GetPhaseRegister(void) ;
#endif /* defined(cADC__CFG3) */

#define cADC_Enable()                       cADC_Start()
#define cADC_Disable()                      cADC_Stop()
#define cADC_SetDivider(clkDivider)         cADC_SetDividerRegister(clkDivider, 1u)
#define cADC_SetDividerValue(clkDivider)    cADC_SetDividerRegister((clkDivider) - 1u, 1u)
#define cADC_SetMode(clkMode)               cADC_SetModeRegister(clkMode)
#define cADC_SetSource(clkSource)           cADC_SetSourceRegister(clkSource)
#if defined(cADC__CFG3)
#define cADC_SetPhase(clkPhase)             cADC_SetPhaseRegister(clkPhase)
#define cADC_SetPhaseValue(clkPhase)        cADC_SetPhaseRegister((clkPhase) + 1u)
#endif /* defined(cADC__CFG3) */


/***************************************
*             Registers
***************************************/

/* Register to enable or disable the clock */
#define cADC_CLKEN              (* (reg8 *) cADC__PM_ACT_CFG)
#define cADC_CLKEN_PTR          ((reg8 *) cADC__PM_ACT_CFG)

/* Register to enable or disable the clock */
#define cADC_CLKSTBY            (* (reg8 *) cADC__PM_STBY_CFG)
#define cADC_CLKSTBY_PTR        ((reg8 *) cADC__PM_STBY_CFG)

/* Clock LSB divider configuration register. */
#define cADC_DIV_LSB            (* (reg8 *) cADC__CFG0)
#define cADC_DIV_LSB_PTR        ((reg8 *) cADC__CFG0)
#define cADC_DIV_PTR            ((reg16 *) cADC__CFG0)

/* Clock MSB divider configuration register. */
#define cADC_DIV_MSB            (* (reg8 *) cADC__CFG1)
#define cADC_DIV_MSB_PTR        ((reg8 *) cADC__CFG1)

/* Mode and source configuration register */
#define cADC_MOD_SRC            (* (reg8 *) cADC__CFG2)
#define cADC_MOD_SRC_PTR        ((reg8 *) cADC__CFG2)

#if defined(cADC__CFG3)
/* Analog clock phase configuration register */
#define cADC_PHASE              (* (reg8 *) cADC__CFG3)
#define cADC_PHASE_PTR          ((reg8 *) cADC__CFG3)
#endif /* defined(cADC__CFG3) */


/**************************************
*       Register Constants
**************************************/

/* Power manager register masks */
#define cADC_CLKEN_MASK         cADC__PM_ACT_MSK
#define cADC_CLKSTBY_MASK       cADC__PM_STBY_MSK

/* CFG2 field masks */
#define cADC_SRC_SEL_MSK        cADC__CFG2_SRC_SEL_MASK
#define cADC_MODE_MASK          (~(cADC_SRC_SEL_MSK))

#if defined(cADC__CFG3)
/* CFG3 phase mask */
#define cADC_PHASE_MASK         cADC__CFG3_PHASE_DLY_MASK
#endif /* defined(cADC__CFG3) */

#endif /* CY_CLOCK_cADC_H */


/* [] END OF FILE */
