/*******************************************************************************
* File Name: cDAC.h
* Version 2.20
*
*  Description:
*   Provides the function and constant definitions for the clock component.
*
*  Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_CLOCK_cDAC_H)
#define CY_CLOCK_cDAC_H

#include <cytypes.h>
#include <cyfitter.h>


/***************************************
* Conditional Compilation Parameters
***************************************/

/* Check to see if required defines such as CY_PSOC5LP are available */
/* They are defined starting with cy_boot v3.0 */
#if !defined (CY_PSOC5LP)
    #error Component cy_clock_v2_20 requires cy_boot v3.0 or later
#endif /* (CY_PSOC5LP) */


/***************************************
*        Function Prototypes
***************************************/

void cDAC_Start(void) ;
void cDAC_Stop(void) ;

#if(CY_PSOC3 || CY_PSOC5LP)
void cDAC_StopBlock(void) ;
#endif /* (CY_PSOC3 || CY_PSOC5LP) */

void cDAC_StandbyPower(uint8 state) ;
void cDAC_SetDividerRegister(uint16 clkDivider, uint8 restart) 
                                ;
uint16 cDAC_GetDividerRegister(void) ;
void cDAC_SetModeRegister(uint8 modeBitMask) ;
void cDAC_ClearModeRegister(uint8 modeBitMask) ;
uint8 cDAC_GetModeRegister(void) ;
void cDAC_SetSourceRegister(uint8 clkSource) ;
uint8 cDAC_GetSourceRegister(void) ;
#if defined(cDAC__CFG3)
void cDAC_SetPhaseRegister(uint8 clkPhase) ;
uint8 cDAC_GetPhaseRegister(void) ;
#endif /* defined(cDAC__CFG3) */

#define cDAC_Enable()                       cDAC_Start()
#define cDAC_Disable()                      cDAC_Stop()
#define cDAC_SetDivider(clkDivider)         cDAC_SetDividerRegister(clkDivider, 1u)
#define cDAC_SetDividerValue(clkDivider)    cDAC_SetDividerRegister((clkDivider) - 1u, 1u)
#define cDAC_SetMode(clkMode)               cDAC_SetModeRegister(clkMode)
#define cDAC_SetSource(clkSource)           cDAC_SetSourceRegister(clkSource)
#if defined(cDAC__CFG3)
#define cDAC_SetPhase(clkPhase)             cDAC_SetPhaseRegister(clkPhase)
#define cDAC_SetPhaseValue(clkPhase)        cDAC_SetPhaseRegister((clkPhase) + 1u)
#endif /* defined(cDAC__CFG3) */


/***************************************
*             Registers
***************************************/

/* Register to enable or disable the clock */
#define cDAC_CLKEN              (* (reg8 *) cDAC__PM_ACT_CFG)
#define cDAC_CLKEN_PTR          ((reg8 *) cDAC__PM_ACT_CFG)

/* Register to enable or disable the clock */
#define cDAC_CLKSTBY            (* (reg8 *) cDAC__PM_STBY_CFG)
#define cDAC_CLKSTBY_PTR        ((reg8 *) cDAC__PM_STBY_CFG)

/* Clock LSB divider configuration register. */
#define cDAC_DIV_LSB            (* (reg8 *) cDAC__CFG0)
#define cDAC_DIV_LSB_PTR        ((reg8 *) cDAC__CFG0)
#define cDAC_DIV_PTR            ((reg16 *) cDAC__CFG0)

/* Clock MSB divider configuration register. */
#define cDAC_DIV_MSB            (* (reg8 *) cDAC__CFG1)
#define cDAC_DIV_MSB_PTR        ((reg8 *) cDAC__CFG1)

/* Mode and source configuration register */
#define cDAC_MOD_SRC            (* (reg8 *) cDAC__CFG2)
#define cDAC_MOD_SRC_PTR        ((reg8 *) cDAC__CFG2)

#if defined(cDAC__CFG3)
/* Analog clock phase configuration register */
#define cDAC_PHASE              (* (reg8 *) cDAC__CFG3)
#define cDAC_PHASE_PTR          ((reg8 *) cDAC__CFG3)
#endif /* defined(cDAC__CFG3) */


/**************************************
*       Register Constants
**************************************/

/* Power manager register masks */
#define cDAC_CLKEN_MASK         cDAC__PM_ACT_MSK
#define cDAC_CLKSTBY_MASK       cDAC__PM_STBY_MSK

/* CFG2 field masks */
#define cDAC_SRC_SEL_MSK        cDAC__CFG2_SRC_SEL_MASK
#define cDAC_MODE_MASK          (~(cDAC_SRC_SEL_MSK))

#if defined(cDAC__CFG3)
/* CFG3 phase mask */
#define cDAC_PHASE_MASK         cDAC__CFG3_PHASE_DLY_MASK
#endif /* defined(cDAC__CFG3) */

#endif /* CY_CLOCK_cDAC_H */


/* [] END OF FILE */
