/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
/*
Settings:
1. EP1 In, EP2 Out.
    Input or Output. USB transfers are host centric; 
    therefore, IN refers to transfers to the host; OUT refers to transfers from the host.
2. EP memory managemnet - Auto mode.
3. CPU clock - Maximum possible clock frequency to get maximum throughput.
*/

#include "project.h"
/*
1 - Initialize components
2 - 
3 - 

*/

    /* Defines for DMA */
#define DMA_BYTES_PER_BURST 2
#define DMA_REQUEST_PER_BURST 1
#define DMA_SRC_BASE (CYDEV_PERIPH_BASE)
#define DMA_DST_BASE (CYDEV_SRAM_BASE)

enum{
    U16_BUFFER_SIZE = 32,
    U8_BUFFER_SIZE  = 2*U16_BUFFER_SIZE,
};

/*buffers used to perform data acquisition and transmission*/
struct samples{
    union buffer1{
        uint16_t bufferMem16[U16_BUFFER_SIZE];
        uint8_t bufferMem8[U8_BUFFER_SIZE];
    }val1;
    union buffer2{
        uint16_t bufferUSB16[U16_BUFFER_SIZE];
        uint8_t bufferUSB8[U8_BUFFER_SIZE];
    }val2;
    uint8 bufferOut[8];
}data;

/* Variable declarations for DMA */ 
/* Move these variable declarations to the top of the function */
uint8 DMA_Chan;
uint8 DMA_TD[2];

/*will have three values:
    0 - idle, 
    1 - pointing to bufferMem, 
    2 - pointing to bufferUSB
*/
uint8 handover_flag = 0; 

/*
pointer that will cycle between the two buffers
*/
uint8 * pUSB = data.val1.bufferMem8;

void init(void);

CY_ISR(DMA_done_flag){
    /*change the value of the handover flag*/
    
    if(handover_flag == 0){
        
        // beginning   
        handover_flag = 1;
        pUSB = data.val1.bufferMem8;
        
    }
    else if(handover_flag == 1){
        //LED_Write(1);
        handover_flag = 2;
        pUSB = data.val2.bufferUSB8;
        //CyDelay(2000);
    }
    else if(handover_flag == 2){  
        //LED_Write(0);
        handover_flag = 1;
        pUSB = data.val1.bufferMem8;
        //CyDelay(2000);
    }
    else
        handover_flag = 0;
}

int main(void)
{
    /* Enable global interrupts. */
    CyGlobalIntEnable;
    
    init();
    
    /* Wait for USB device to enumerate */
    while(!USB_bGetConfiguration());
    
    /* Enumeration is done, enable OUT endpoint (EP2) for receive data from Host */
    USB_EnableOutEP(2);
    
    /* wait until we have data */
        while(USB_GetEPState(2) == USB_OUT_BUFFER_EMPTY);
        /* Get the data */     
        USB_ReadOutEP(2, data.bufferOut, 1);
    
    DMA_DONE_StartEx(DMA_done_flag);
    
    for(;;){
 
       if(handover_flag != 1 || handover_flag != 2)
            continue;
        
        USB_EnableOutEP(2);
        LED_Write(1);
        while(USB_GetEPState(1) != USB_IN_BUFFER_EMPTY){} // Wait until our IN EP is empty
        USB_LoadInEP(1, pUSB, U8_BUFFER_SIZE);
      
    }
}
/*
Initialization of the PSoC's components
*/
void init(void){
    
    
    USB_Start(0, USB_5V_OPERATION);   /*Start USBFS*/
    ADC_Start();                      /*Start ADC*/
    adc_clk_Start();                  /*Start ADC's clock*/
    DAC_Start();                      /*Start DAC*/
    dac_clk_Start();                  /*Start DAC's clock*/
    //initialize Multiplexer

    /* DMA Configuration for DMA */
    DMA_Chan = DMA_DmaInitialize(DMA_BYTES_PER_BURST, DMA_REQUEST_PER_BURST, 
        HI16(DMA_SRC_BASE), HI16(DMA_DST_BASE));
    DMA_TD[0] = CyDmaTdAllocate();
    DMA_TD[1] = CyDmaTdAllocate();
    CyDmaTdSetConfiguration(DMA_TD[0], 32, DMA_TD[1], DMA__TD_TERMOUT_EN | CY_DMA_TD_INC_DST_ADR);
    CyDmaTdSetConfiguration(DMA_TD[1], 32, DMA_TD[0], DMA__TD_TERMOUT_EN | CY_DMA_TD_INC_DST_ADR);
    CyDmaTdSetAddress(DMA_TD[0], LO16((uint32)ADC_DEC_SAMP_PTR), LO16((uint32)&data.val1.bufferMem16[0]));
    CyDmaTdSetAddress(DMA_TD[1], LO16((uint32)ADC_DEC_SAMP_PTR), LO16((uint32)&data.val2.bufferUSB16[0]));
    CyDmaChSetInitialTd(DMA_Chan, DMA_TD[0]);
    CyDmaChEnable(DMA_Chan, 1);
    
    /* DMA Configuration for DMA */
    /*
    DMA_Chan = DMA_DmaInitialize(DMA_BYTES_PER_BURST, DMA_REQUEST_PER_BURST, 
        HI16(DMA_SRC_BASE), HI16(DMA_DST_BASE));
    DMA_TD[0] = CyDmaTdAllocate();
    DMA_TD[1] = CyDmaTdAllocate();
    CyDmaTdSetConfiguration(DMA_TD[0], 32, DMA_TD[1], CY_DMA_TD_INC_DST_ADR);
    CyDmaTdSetConfiguration(DMA_TD[1], 32, CY_DMA_DISABLE_TD, DMA__TD_TERMOUT_EN | CY_DMA_TD_INC_DST_ADR);
    CyDmaTdSetAddress(DMA_TD[0], LO16((uint32)ADC_DEC_SAMP_PTR), LO16((uint32)&data.val1.bufferMem16[0]));
    CyDmaTdSetAddress(DMA_TD[1], LO16((uint32)ADC_DEC_SAMP_PTR), LO16((uint32)&data.val2.bufferUSB16[0]));
    CyDmaChSetInitialTd(DMA_Chan, DMA_TD[0]);
    CyDmaChEnable(DMA_Chan, 1);
    */
}
/* [] END OF FILE */

